//This program is licensed under the GPLv3; see license.txt for more info.
//Copyright 2009 Sprite_tm

#include "io.h"
#include "sequence.h"
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

//Wait till a button has been pressed or the timeout
//has passed. Returns the button value or -1 on timeout.
short getButton(int timeout) {
    short x;
    short btn=-1;
    while (timeout!=0 && btn==-1) {
	for (x=0; x<9; x++) {
    	    if (pressedButtons&(1<<x)) btn=x;
	}
	timeout--;
	_delay_us(750);
    }
    return btn;
}


//Program a sequence
void progSeq() {
    int pos, x;
    char screenIsNew;
    short b;
    screenType screen;
    
    pos=newSeqStart();
    do {
	//Initialize a screen, clear all leds
	screenIsNew=1;
	screen.flags=0;
	for (x=0; x<9; x++) {
	    ledIntensity[x]=0;
	    screen.led[x]=SCREEN_LED_UNUSED;
	}
	
	//Program a screen.
	b=getButton(2000);
	while (b!=-1) {
	    screenIsNew=0; //Screen is tainted now ;)
	    //We have a button. See if we need to turn the corresponding led on or
	    //off.
	    if (ledIntensity[b]==0) {
		//Add led to array, turn it on
		x=0; 
		while (screen.led[x]!=SCREEN_LED_UNUSED) x++;
		screen.led[x]=b;
		ledIntensity[b]=255;
	    } else {
		//Remove led from array, turn it off
		char found=0;
		for (x=0; x<9; x++) {
		    if (screen.led[x]==b) found=1;
		    if (found && x!=8) screen.led[x]=screen.led[x+1];
		}
		screen.led[8]=SCREEN_LED_UNUSED;
		ledIntensity[b]=0;
	    }
	    //Wait for user to release button
	    _delay_ms(600);
	    //Grab another button.
	    b=getButton(2000);
	}
	//If a new screen has timeouted, the user hasn't pressed anything for
	//4 secondsm which indicates the end of a sequence.
	if (screenIsNew) screen.flags=SCREEN_FLAGS_END;
	//Write screen to eeprom.
	writeScreen(&screen, pos);
	//Next screen :)
	pos+=SCREEN_LEN;
    } while (!screenIsNew);
}


//If a led isn't completely turned off, this will turn it on a bit more.
void turnOnMore() {
    int x;
    for (x=0; x<9; x++) {
	if (ledIntensity[x]!=0 && ledIntensity[x]!=255) ledIntensity[x]++;
	//Somehow, 127->128 looks iffy because of the jump from 0b011111 to 0b10000.
	//The next line introduces an artificial jump which actually makes it look
	//better.
	if (ledIntensity[x]==100) ledIntensity[x]=128;
    }
}

//Main routine
int main(void) {
    short x,y;
    short pos;
    int btn;
    screenType screen;
    initIo();
    sei();

    //Small init loop Just light and dim all leds to get any 
    //power-on transients out of the system.
    for (x=0; x<9; x++) ledIntensity[x]=0;
    while(ledIntensity[1]<128) {
	for (x=0; x<9; x++) ledIntensity[x]++;
	_delay_ms(1);
    }
    while(ledIntensity[1]>0) {
	for (x=0; x<9; x++) ledIntensity[x]--;
	_delay_ms(1);
    }
    _delay_ms(1000); //sleep a bit more
    //User should be not touching the device now, so let's calibrate the buttons.
    resetCal();

    //Go display!
    while(1) {
	pos=getOldestSeq();
	//Start displaying
	while(pressedButtons==0) {
	    //Fade out all leds
	    for (y=255; y>0; y--) {
		for (x=0; x<9; x++) {
		    if (ledIntensity[x]!=0) ledIntensity[x]--;
		}
		_delay_ms(1);
	    }
	    getButton(300);
	    //Read a screen from storage
	    readScreen(&screen, pos);
	    //Display leds in order
	    for (y=0; y<9 && pressedButtons==0; y++) {
		if (screen.led[y]!=SCREEN_LED_UNUSED) {
		    //Fade in that led
		    ledIntensity[screen.led[y]]=1;
		    for (x=0; x<32; x++) {
			turnOnMore();
			_delay_ms(8);
		    }
		}
	    }
	    if (screen.led[y]!=SCREEN_LED_UNUSED) {
		//Finish fading
		for (x=0; x<255 && pressedButtons==0; x++) {
    		    turnOnMore();
		    _delay_ms(8);
		}
	    }
	    getButton(1000);
	    if (screen.flags==SCREEN_FLAGS_END) {
		//End of screen. Switch for next turn.
		pos=getOtherSeq(pos);
	    } else {
		//Next screen.
		pos+=SCREEN_LEN;
	    }
	}
	progSeq();
    }
}
