#include <sequence.h>
#include <avr/eeprom.h>

//For space reasons, the EEPROM is allocated in nibbles. One screen consists of
//10 nibbles: one storing flags and the remaining 9 storing the order of LEDS
//to turn on. This totals 5 bytes.
//Sequences get prefixed by a sequence ID which gets one higher every time a 
//sequence is overwritten. This way, the AVR knows which sequence is the older
//one and which is the newer one.

#define SEQA_START 0
#define SEQB_START 256
#define SEQA_DATA 1
#define SEQB_DATA 257

//Parse 5 bytes of eeprom into 10 nibbles into the screen struct.
void readScreen(screenType *screenbuff, int pos) {
    unsigned char b;
    b=eeprom_read_byte(pos++);
    screenbuff->flags=(b>>4)&0xf; 
    screenbuff->led[0]=(b&0xf);
    b=eeprom_read_byte(pos++);
    screenbuff->led[1]=(b>>4)&0xf; 
    screenbuff->led[2]=(b&0xf);
    b=eeprom_read_byte(pos++);
    screenbuff->led[3]=(b>>4)&0xf; 
    screenbuff->led[4]=(b&0xf);
    b=eeprom_read_byte(pos++);
    screenbuff->led[5]=(b>>4)&0xf; 
    screenbuff->led[6]=(b&0xf);
    b=eeprom_read_byte(pos++);
    screenbuff->led[7]=(b>>4)&0xf; 
    screenbuff->led[8]=(b&0xf);
}

//Convert the screen struct to 100 nibbles and write them to eeprom.
void writeScreen(screenType *screenbuff, int pos) {
    eeprom_write_byte(pos++, (screenbuff->flags<<4)+screenbuff->led[0]);
    eeprom_write_byte(pos++, (screenbuff->led[1]<<4)+screenbuff->led[2]);
    eeprom_write_byte(pos++, (screenbuff->led[3]<<4)+screenbuff->led[4]);
    eeprom_write_byte(pos++, (screenbuff->led[5]<<4)+screenbuff->led[6]);
    eeprom_write_byte(pos++, (screenbuff->led[7]<<4)+screenbuff->led[8]);
}

//This gets passed a position in EEPROM and will return the start of the 
//'other' sequence.
int getOtherSeq(int pos) {
    if (pos>=SEQB_DATA) pos=SEQA_DATA; else pos=SEQB_DATA;
    return pos;
}

//Return a pointer to the oldest of the two sequences.
int getOldestSeq() {
    char a, b;
    a=eeprom_read_byte(SEQA_START);
    b=eeprom_read_byte(SEQB_START);
    if (a>b || (a==0 && b==0xff)) {
	return SEQA_DATA;
    } else {
	return SEQB_DATA;
    }
}

//Start programming a new sequence.
int newSeqStart() {
    char d;
    int oldseq;
    int newseq;
    //read newest seq which will become the old seq
    if (getOldestSeq()==SEQB_DATA) oldseq=SEQB_START; else oldseq=SEQA_START;
    d=eeprom_read_byte(oldseq);
    d++; //next seq number
    //write newer seq no
    if (getOldestSeq()==SEQB_DATA) newseq=SEQA_START; else newseq=SEQB_START;
    eeprom_write_byte(newseq, d);
    return newseq+1;
}

